// Class D3iVector:  A 3 dimensional vector class with integer type
// Interface:
//	Constructor:  Default as 0 vector
//		With three float values for components
//
// 	Operations:
//      def constructor as 0 vector
//	    constructor with 3 floats given
//		dot product:  dot
//		add another vector to this one:  addto  
//		cross product:  cross
//		norm
//		scale by factor:  scale
//		Convert to array:   toArray
class D3iVector {
	protected int x[] = {0,0,0};   // components
	
	// Constructors
	D3iVector () {
		}

	D3iVector (int xv, int yv, int zv) {
		x[0] = xv; x[1] = yv; x[2] = zv; 
		}
	
	D3iVector (D3iVector v) {  // copy constructor
		setValue(v);
		}

	boolean equals (D3iVector rhs) {
		for (int i=0; i<3; i++)
			if (x[i] != rhs.x[i])
				return false;
		return true;
		}

	void setValue (int xv, int yv, int zv) {
		x[0] = xv; x[1] = yv; x[2] = zv; 
		}
		
	void setValue (int index, int val) {
		x[index] = val;
		}

	void setValue (int [] v) {
		for (int i=0; i<3; i++)
			x[i] = v[i];
		}

	void setValue (D3iVector xv) {
		for (int i=0; i<3; i++)
			x[i] = xv.x[i];
		}

	int getValue(int index) {
		return x[index];
		}
		
	int x() { return x[0];}
	int y() { return x[1];}
	int z() { return x[2];}
	
	synchronized double norm() {
		double sum = 0;
		for (int i=0; i<3; i++) 
			sum += x[i]*x[i];
		return Math.sqrt(sum);
		}
	
	
	// Return sum
	D3iVector add(D3iVector Rhs) {
		D3iVector temp = new D3iVector(Rhs);
		addto(temp);
		return temp;
	
	}

	void addto (D3iVector Rhs) {
		for (int i=0; i<3; i++)
			x[i] += Rhs.x[i];
		}
		
	int dot(D3iVector Rhs) {
		int sum=0;
		for (int i=0; i<3; i++)
			sum += x[i] * Rhs.x[i];
		return sum;
		}

	// Find the average of this and v1 and put in target.
	// used for finding edge cubie indices from corner vertices.
	void interpolate (D3iVector v1, D3iVector target)  {
		for (int i=0; i<3; i++)
			target.setValue(i, (x[i]+v1.x[i])/2);
		}

	void cross(D3iVector Rhs, D3iVector Target) {
		if (Target == null) 
			Target = new D3iVector();
		int xr = Rhs.x[0], yr = Rhs.x[1], zr = Rhs.x[2];
		Target.x[0] = x[1] * zr - x[2] * yr;
		Target.x[1] = x[2] * xr - x[0] * zr;
		Target.x[2] = x[0] * yr - x[1] * xr;
		}
		
    void scale(int fv) {
    	for(int i=0; i<3; i++)
    		x[i] *= fv;
		}
	
    public String toString() {
		return ("["  + x[0] + "," + x[1] + "," + x[2] + "]");
	    }

}  // end of D3iVector


// Class D3iMatrix:  A 3 dimensional matrix class with integers.
//    	This class is meant to manage 90 degree rotations so only +/- 1 and 0
// 		should appear in matrix
// Interface:
// 		Constructor:  Default as identity
// 			With three vectors as columns or rows (determined by boolean param).
//			Set matrix to rotation about a face (0 thru 5):  setRotation(face)
//			multiply and change a 3 vec:  multSetV
//			set to unit:  unit()
//			set to zero:  zero()
//			toString()
//

class D3iMatrix {
	protected int c[][] = new int [3][3]; // Row, column
//    static final double pi = 3.14159265;
    
    /** Create a new unit matrix */
    // assumes other values initialized to 0
    D3iMatrix () {
    	for (int i=0; i<3; i++) 
    		c[i][i] = 1;
	    }
	
	D3iMatrix(int face) { // Initialize to a rotation about this face.
		this();
		setRotation(face);
		}
		
	// set matrix with three vectors
	// if rows is true, they are taken as the rows.
	// otherwise, taken as the columns
	D3iMatrix( D3iVector xv, D3iVector yv, D3iVector zv, boolean rows)  {
		int i,j;
		if (rows) {
			for (i = 0; i<3; i++) {
				c[0][i] = xv.x[i];
				c[1][i] = yv.x[i];
				c[2][i] = zv.x[i];
				}
			}
		else {
			for (i = 0; i<3; i++) {
				c[i][0] = xv.x[i];
				c[i][1] = yv.x[i];
				c[i][2] = zv.x[i];
				}
			}
		} // end of constructor with vectors

	void setValue(int i, int j, int value) {
		c[i][j] = value;
		}


	void setValue( D3iVector xv, D3iVector yv, D3iVector zv, boolean rows)  {
		int i,j;
		if (rows) {
			for (i = 0; i<3; i++) {
				c[0][i] = xv.x[i];
				c[1][i] = yv.x[i];
				c[2][i] = zv.x[i];
				}
			}
		else {
			for (i = 0; i<3; i++) {
				c[i][0] = xv.x[i];
				c[i][1] = yv.x[i];
				c[i][2] = zv.x[i];
				}
			}
		} // end of set value with vectors

	// set a row or column of the matrix
	void setValue (int index, int xv, int yv, int zv, boolean arow) {
		if (arow) {
			c[index][0] = xv;
			c[index][1] = yv;
			c[index][2] = zv;
			}
		else {
			c[0][index] = xv;
			c[1][index] = yv;
			c[2][index] = zv;
			}
		}
	
	int getValue(int i, int j) {
		return c[i][j];
		}
	
	D3iVector Row(int i) {
		D3iVector temp = new D3iVector(c[i][0], c[i][1], c[i][2]);
		return temp;
		}
		
	// Put col i into target
	D3iVector Column(int i)  {
		D3iVector temp = new D3iVector(c[0][i], c[1][i], c[2][i]);
		return temp;
		}

	// Put row i into target
	void getRow(int i, D3iVector Target)  {
		Target.setValue(c[i][0], c[i][1], c[i][2]);
		}

	// Put col i into target
	void getColumn(int i, D3iVector Target)  {
		Target.setValue(c[0][i], c[1][i], c[2][i]);
		}
	    
    /** Multiply this matrix by a second on the right: M = M*R */
    void Rmult(D3iMatrix rhs) {
    
    	D3iMatrix result = new D3iMatrix();
    	D3iVector arow = new D3iVector();
    	int rowidx,column, i, j;
    	
    	result.zero();
    	for (rowidx=0; rowidx<3; rowidx++)  
    		for (column=0; column<3; column++) 
    			// Find the row, column entry
    			for (i=0; i<3; i++)
    				result.c[rowidx][column] += c[rowidx][i] * rhs.c[i][column];

		// Copy the result into this
    	for (i=0; i<3; i++)
    		for (j=0; j<3; j++)
				c[i][j] = result.c[i][j];
		
	    }  // end mult
    
    /** Multiply this matrix by a second: M = L*M */
    // Left sided multiplication.
    void Lmult(D3iMatrix lhs) {
    
    	D3iMatrix result = new D3iMatrix();
    	int rowidx,column, i, j;
    	
    	
    	result.zero();
    	for (rowidx=0; rowidx<3; rowidx++)  
    		for (column=0; column<3; column++) 
    			// Find the row, column entry
    			for (i=0; i<3; i++)
    				result.c[rowidx][column] += lhs.c[rowidx][i] * c[i][column];

		// Copy the result into this
    	for (i=0; i<3; i++)
    		for (j=0; j<3; j++)
				c[i][j] = result.c[i][j];
		
	    }  // end Lmult
    

	
    /** Set this matrix to a 90 degree rotation about the given axis  */
    // 0 is x_0, 3 is x_1, etc.
    // Assumes this matrix will be multiplying on the left to produce rotation.
    void setRotation(int face)   {
    	int axis1=0, axis2=0; // the other (moving) axes.
    	
    	unit();  // Just to be safe.
    	
    	// For the other variables (not face%3) set the location to the
    	// usual rotation matrix.
    	if (face==0 || face == 5) axis1 = 1;
    	else if (face==1 || face == 3) axis1 = 2;
    	if (face==0 || face==4) axis2 = 2;
    	else if (face==2 || face==3) axis2 = 1;

    	c[axis1][axis1] = 0;
    	c[axis2][axis2] = 0;
    	c[axis1][axis2] =  -1;
    	c[axis2][axis1] = 1;
	    }
    // Set this matrix to a 90 degree rotation moving the axes
    // passed as parameters.
    // 0 is x_0, 1 is x_1, etc.
    // Assumes this matrix will be multiplying on the left to produce rotation.
    void setRotation(int axis1, int axis2)   {
    	
    	unit();  // Just to be safe.
    	
    	c[axis1][axis1] = 0;
    	c[axis2][axis2] = 0;
    	c[axis1][axis2] =  -1;
    	c[axis2][axis1] = 1;
	    }

	D3iVector mult(D3iVector v)  {
    	D3iVector temp = new D3iVector();
    	int sum = 0;
    	int i,j;
    	for (i = 0; i<3; i++) {
    		sum = 0;
       		for (j=0; j< 3; j++)
       			sum += c[i][j] * v.x[j];
       		temp.setValue(i,sum);
       		}
       	return temp;
       	}

     // Multiply a this matrix times a 3 vector and alter the 3 vector
    void multSetV(D3iVector v)  {
    	
    	D3iVector temp = new D3iVector();
    	int sum = 0;
    	int i,j;
    	for (i = 0; i<3; i++) {
    		sum = 0;
       		for (j=0; j< 3; j++)
       			sum += c[i][j] * v.x[j];
       		temp.setValue(i,sum);
       		}
       	v.setValue(temp);
		}  // end multSetV
	
		
	void zero() {  // set mat to 0 all entries
    	int i,j;
    	for (i=0; i<3; i++)
    		for (j=0; j<3; j++)
    			c[i][j] = 0;
	
	}
	
   /** Reinitialize to the unit matrix */
    void unit() {
    	
    	int i,j; 
    	
		zero();
		for (i=0; i<3; i++)
			c[i][i] = 1;
	    }  // end unit

	// return true if matrix is identity.
	boolean isunit() {
		int i,j;
		for (i=0; i<3; i++) {
			if (c[i][i] != 1) return false;
			for (j = 0; j<3; j++) 
				if (i != j && c[i][j] != 0) return false;
			}
		return true;
		}
			
    public String toString() {
	return ("[" + c[0][0] + "," + c[0][1] + "," + c[0][2] + ";\n"
		 + c[1][0] + "," + c[1][1] + "," + c[1][2] + ";\n"
		 + c[2][0] + "," + c[2][1] + "," + c[2][2] + "]");
    }



}  // end of class D3iMatrix