

// Class D4iVector:  A 4 dimensional vector class with integer type
// Interface:
//	Constructor:  Default as 0 vector
//		With three float values for components
//
// 	Operations:
//      def constructor as 0 vector
//	    constructor with 4 integers given
//		dot product:  dot
//		add another vector to this one:  addto  
//		cross product:  cross
//		norm
//		scale by factor:  scale
//		Convert to array:   toArray
class D4iVector {
	protected int x[] = {0,0,0,0};   // components
	
	// Constructors
	D4iVector () {
		}

	D4iVector (int x0, int x1, int x2, int x3) {
		x[0] = x0; x[1] = x1; x[2] = x2; x[3] = x3;
		}
	
	D4iVector (D4iVector v) {  // copy constructor
		setValue(v);
		}

	boolean equals (D4iVector rhs) {
		for (int i=0; i<4; i++)
			if (x[i] != rhs.x[i])
				return false;
		return true;
		}

	void setValue  (int x0, int x1, int x2, int x3) {
		x[0] = x0; x[1] = x1; x[2] = x2; x[3] = x3;
		}
	
	// No error checking.
	void setValue (int index, int val) {
		x[index] = val;
		}

	void setValue (int [] v) {
		for (int i=0; i<4; i++)
			x[i] = v[i];
		}

	void setValue (D4iVector xv) {
		for (int i=0; i<4; i++)
			x[i] = xv.x[i];
		}

	int getValue(int index) {
		return x[index];
		}

	synchronized double norm() {
		double sum = 0;
		for (int i=0; i<4; i++) 
			sum += x[i]*x[i];
		return Math.sqrt(sum);
		}
	
	
	// Return sum
	D4iVector add(D4iVector Rhs) {
		D4iVector temp = new D4iVector(Rhs);
		addto(temp);
		return temp;
	
	}

	void addto (D4iVector Rhs) {
		for (int i=0; i<4; i++)
			x[i] += Rhs.x[i];
		}
		
	int dot(D4iVector Rhs) {
		int sum=0;
		for (int i=0; i<4; i++)
			sum += x[i] * Rhs.x[i];
		return sum;
		}

    void scale(int fv) {
    	for(int i=0; i<4; i++)
    		x[i] *= fv;
		}
	
    public String toString() {
		return ("["  + x[0] + "," + x[1] + "," + x[2] + "," + x[3] + "]");
	    }

	// Project from a 4 vector to a 3 vector.  
	// Remove the index passed as a parameter.
	D3iVector projection(int index)  {
		D3iVector temp = new D3iVector ();
		int i,j=0;
		for ( i=0; i<4; i++)  
			if (i != index) temp.setValue(j++,x[i]);
		return temp;
		} // End of projection function.
		
	// Project from a 4 vector to a 3 vector.  
	// Project the three indices passed in the order passed.
	D3iVector projection(int i0, int i1, int i2)  {
		D3iVector temp = new D3iVector (x[i0],x[i1],x[i2]);
		return temp;
		} // End of projection function.
		
	// Project from a 4 vector to a 3 vector.  
	// Remove the index passed as a parameter.
	void projection(int index, D3iVector target)  {
		int i,j=0;
		for ( i=0; i<4; i++)  
			if (i != index) target.setValue(j++,x[i]);
		} // End of projection function.
		
	// Project from a 4 vector to a 3 vector.  
	// Project the three indices passed in the order passed.
	void projection(int i0, int i1, int i2, D3iVector target)  {
		target.setValue(x[i0],x[i1],x[i2]);
		} // End of projection function.
		
}  // end of D4iVector


// Class D4iMatrix:  A 4 dimensional matrix class with integers.
//    	This class is meant to manage 90 degree rotations so only +/- 1 and 0
// 		should appear in matrix
// Interface:
// 		Constructor:  Default as identity
// 			With three vectors as columns or rows (determined by boolean param).
//			Set matrix to rotation about a face (0 thru 5):  setRotation(face)
//			multiply and change a 4 vec:  multSetV
//			set to unit:  unit()
//			set to zero:  zero()
//			toString()
//

class D4iMatrix {
	protected int c[][] = new int [4][4]; // Row, column
//    static final double pi = 3.14159265;
    
    /** Create a new unit matrix */
    // assumes other values initialized to 0
    D4iMatrix () {
    	for (int i=0; i<4; i++) 
    		c[i][i] = 1;
	    }
	
	// Initialize to a rotation about this face.
	// NOTE:  PARAMETERS ARE THE MOVING AXES, NOT THE 
	// AXES ABOUT WHICH WE ROTATE.
	D4iMatrix(int axis1, int axis2) { 
		this();
		setRotation(axis1, axis2);
		}

	void setValue(int i, int j, int value) {
		c[i][j] = value;
		}

		
	// set matrix with three vectors
	// if rows is true, they are taken as the rows.
	// otherwise, taken as the columns
	D4iMatrix( D4iVector x0, D4iVector x1, D4iVector x2,  D4iVector x3, boolean rows)  {
		int i,j;
		if (rows) {
			for (i = 0; i<4; i++) {
				c[0][i] = x0.x[i];
				c[1][i] = x1.x[i];
				c[2][i] = x2.x[i];
				c[3][i] = x3.x[i];
				}
			}
		else {
			for (i = 0; i<4; i++) {
				c[i][0] = x0.x[i];
				c[i][1] = x1.x[i];
				c[i][2] = x2.x[i];
				c[i][3] = x3.x[i];
				}
			}
		} // end of constructor with vectors

	void setValue(  D4iVector x0, D4iVector x1, D4iVector x2,  D4iVector x3, boolean rows)  {
		int i,j;
		if (rows) {
			for (i = 0; i<4; i++) {
				c[0][i] = x0.x[i];
				c[1][i] = x1.x[i];
				c[2][i] = x2.x[i];
				c[3][i] = x3.x[i];
				}
			}
		else {
			for (i = 0; i<4; i++) {
				c[i][0] = x0.x[i];
				c[i][1] = x1.x[i];
				c[i][2] = x2.x[i];
				c[i][3] = x3.x[i];
				}
			}
		} // end of set value with vectors

	// set a row or column of the matrix
	void setValue (int index, int x0, int x1, int x2, int x3, boolean arow) {
		if (arow) {
			c[index][0] = x0;
			c[index][1] = x1;
			c[index][2] = x2;
			c[index][3] = x3;
			}
		else {
			c[0][index] = x0;
			c[1][index] = x1;
			c[2][index] = x2;
			c[3][index] = x3;
			}
		}
	
	// No error checking
	int getValue(int i, int j) {
		return c[i][j];
		}
	
	D4iVector Row(int i) {
		D4iVector temp = new D4iVector(c[i][0], c[i][1], c[i][2], c[i][3]);
		return temp;
		}
		
	// Put col i into target
	D4iVector Column(int i)  {
		D4iVector temp = new D4iVector(c[0][i], c[1][i], c[2][i], c[3][i]);
		return temp;
		}

	// Put row i into target
	void getRow(int i, D4iVector Target)  {
		Target.setValue(c[i][0], c[i][1], c[i][2], c[i][3]);
		}

	// Put col i into target
	void getColumn(int i, D4iVector Target)  {
		Target.setValue(c[0][i], c[1][i], c[2][i], c[3][i]);
		}
	    
    /** Multiply this matrix by a second on the right: M = M*R */
    void Rmult(D4iMatrix rhs) {
    
    	D4iMatrix result = new D4iMatrix();
    	D4iVector arow = new D4iVector();
    	int rowidx,column, i, j;
    	
    	result.zero();
    	for (rowidx=0; rowidx<4; rowidx++)  
    		for (column=0; column<4; column++) 
    			// Find the row, column entry
    			for (i=0; i<4; i++)
    				result.c[rowidx][column] += c[rowidx][i] * rhs.c[i][column];

		// Copy the result into this
    	for (i=0; i<4; i++)
    		for (j=0; j<4; j++)
				c[i][j] = result.c[i][j];
		
	    }  // end mult
    
    /** Multiply this matrix by a second: M = L*M */
    // Left sided multiplication.
    void Lmult(D4iMatrix lhs) {
    
    	D4iMatrix result = new D4iMatrix();
    	int rowidx,column, i, j;
    	
    	
    	result.zero();
    	for (rowidx=0; rowidx<4; rowidx++)  
    		for (column=0; column<4; column++) 
    			// Find the row, column entry
    			for (i=0; i<4; i++)
    				result.c[rowidx][column] += lhs.c[rowidx][i] * c[i][column];

		// Copy the result into this
    	for (i=0; i<4; i++)
    		for (j=0; j<4; j++)
				c[i][j] = result.c[i][j];
		
	    }  // end Lmult
    

	// NOTE MAJOR CHANGE FROM C3:  THIS ROUTINE
	// REQUIRES INDICATING WHICH AXES ARE TO BE MOVED, NOT
	// WHICH ARE FIXED.  This has been added as an overloaded 
	// function for C3.
	//
    // Set this matrix to a 90 degree 
    // 0 is x_0, 1 is x_1, etc.
    // The 2x2 matrix looks like:  
    // 0 -1
    // 1  0    (more generally, sin(t) for 1 and cos(t) for 0.)
    // Assumes this matrix will be multiplying on the left to produce rotation.
    // Both face1 axis and face2 axis are held rigid.
    void setRotation(int axis1, int axis2)   {
    	
    	unit();  

    	c[axis1][axis1] = 0;
    	c[axis2][axis2] = 0;
    	c[axis1][axis2] =  -1;
    	c[axis2][axis1] = 1;
	    }

	D4iVector mult(D4iVector v)  {
    	D4iVector temp = new D4iVector();
    	int sum = 0;
    	int i,j;
    	for (i = 0; i<4; i++) {
    		sum = 0;
       		for (j=0; j< 4; j++)
       			sum += c[i][j] * v.x[j];
       		temp.setValue(i,sum);
       		}
       	return temp;
       	}

     // Multiply a this matrix times a 4 vector and alter the 4 vector
    void multSetV(D4iVector v)  {
    	
    	D4iVector temp = new D4iVector();
    	int sum = 0;
    	int i,j;
    	for (i = 0; i<4; i++) {
    		sum = 0;
       		for (j=0; j< 4; j++)
       			sum += c[i][j] * v.x[j];
       		temp.setValue(i,sum);
       		}
       	v.setValue(temp);
		}  // end multSetV
	
		
	void zero() {  // set mat to 0 all entries
    	int i,j;
    	for (i=0; i<4; i++)
    		for (j=0; j<4; j++)
    			c[i][j] = 0;
	
	}
	
   /** Reinitialize to the unit matrix */
    void unit() {
    	
    	int i,j; 
    	
		zero();
		for (i=0; i<4; i++)
			c[i][i] = 1;
	    }  // end unit

	// Project this matrix to a 3x3 matrix removing teh
	// indicated axis.  There are clever ways to do this,
	// but it I'm doing it pretty directly.
	// PRECONDITION:  axis is between 0 and 3.
	void projection(int axis, D3iMatrix target) {
	
		int i,j;  // row and column for this
		int row=0, column=0; // for target
		for (i=0; i<4; i++)
			if (i != axis) {
				for (j=0; j<4; j++)
					if (j != axis) 
						target.setValue(row, column++, c[i][j]);
				column = 0;
				row += 1;
				}
		}
			

/*
	// Project this matrix to a 3x3 matrix removing teh
	// indicated axis.  There are clever ways to do this,
	// but it I'm doing it with a little bit of speed in mind.
	void projection(D3iMatrix Mat, int axis)  {
		int i,j;  // indices for this
		int iM=0, jM=0;   // indices for M
//		if (M==null) M = new D3iMatrix();
		for (i=0; i<4; i++)  // For each row in this,
			if (i != axis) {
				for (j=0; j<4; j++)  // for each column
					if (j != axis) 
						Mat.setValue(iM, jM++, c[i][j])
				jM = 0; ++iM;
				}
	} // end of projection function.
*/
	// return true if matrix is identity.
	boolean isunit() {
		int i,j;
		for (i=0; i<4; i++) {
			if (c[i][i] != 1) return false;
			for (j = 0; j<4; j++) 
				if (i != j && c[i][j] != 0) return false;
			}
		return true;
		}
			
    public String toString() {
	return ("[" + c[0][0] + "," + c[0][1] + "," + c[0][2] + "," + c[0][3] + ";\n"
		 + c[1][0] + "," + c[1][1] + "," + c[1][2] + "," + c[1][3] + ";\n"
		 + c[2][0] + "," + c[2][1] + "," + c[2][2] + "," + c[2][3] + "\n"
		 + c[3][0] + "," + c[3][1] + "," + c[3][2] + "," + c[3][3] + "]");
    }



}  // end of class D4iMatrix