/* THIS FILE MODIFIED BY DANIEL TALAYCO FROM:
 * Sun's Matrix3D.java	1.2 96/12/06
 * It no longer bears any resemblance to the original.
 */

// Modified by Dan Talayco, 2/98
// For vector and matrix 3 D classes.
//
//
// Class D3Vector:  A 3 dimensional vector class.
// Interface:
//	Constructor:  Default as 0 vector
//		With three float values for components
//
// 	Operations:
//      def constructor as 0 vector
//	    constructor with 3 floats given
//		dot product:  dot
//		add another vector to this one:  addto  
//		cross product:  cross
//		norm
//		scale by factor:  scale
//		Convert to array:   toArray
//	

class D3Vector {
	protected float x[] = {0,0,0};   // components
	
	// Constructors
	D3Vector () {
		}
	D3Vector (float xv, float yv, float zv) {
		x[0] = xv; x[1] = yv; x[2] = zv; 
		}
	
	D3Vector (D3Vector v) {  // copy constructor
		setValue(v);
		}

	boolean equals (D3Vector rhs) {
		for (int i=0; i<3; i++)
			if (x[i] != rhs.x[i])
				return false;
		return true;
		}

	void setValue (float xv, float yv, float zv) {
		x[0] = xv; x[1] = yv; x[2] = zv; 
		}
		
	void setValue (int index, float val) {
		x[index] = val;
		}

	void setValue (D3Vector xv) {
		for (int i=0; i<3; i++)
			x[i] = xv.x[i];
		}

	void setValue (D3iVector xv) {
		for (int i=0; i<3; i++)
			x[i] = xv.getValue(i);
		}
		
	void setValue (float [] v) {
		for (int i=0; i<3; i++)
			x[i] = v[i];
		}

	float getValue(int index) {
		return x[index];
		}

	synchronized double norm() {
		double sum = 0;
		for (int i=0; i<3; i++) 
			sum += x[i]*x[i];
		return Math.sqrt(sum);
		}
	
	// Return sum
	D3Vector add(D3Vector Rhs) {
		D3Vector temp = new D3Vector(Rhs);
		addto(temp);
		return temp;
	
	}


	void addto (D3Vector Rhs) {
		for (int i=0; i<3; i++)
			x[i] += Rhs.x[i];
		}
		
	float dot(D3Vector Rhs) {
		float sum=0;
		for (int i=0; i<3; i++)
			sum += x[i] * Rhs.x[i];
		return sum;
		}
	
	void cross(D3Vector Rhs, D3Vector Target) {
		if (Target == null) 
			Target = new D3Vector();
		float xr = Rhs.x[0], yr = Rhs.x[1], zr = Rhs.x[2];
		Target.x[0] = x[1] * zr - x[2] * yr;
		Target.x[1] = x[2] * xr - x[0] * zr;
		Target.x[2] = x[0] * yr - x[1] * xr;
		}
		
    void scale(float fv) {
    	for(int i=0; i<3; i++)
    		x[i] *= fv;
		}
	
    public String toString() {
		return ("["  + x[0] + "," + x[1] + "," + x[2] + "]");
	    }

}  // end of D3Vector




// Class D3Matrix:  A 3 dimensional matrix class.
// Interface: 
// 		Constructor:  Default as identity
// 			With three vectors as columns or rows (determined by boolean param).
//			scale by one value or scale x, y, z columns independently
//			rotate about x, y, z:  xrot, yrot, zrot
//			multiply a 3 vec and return 3 vec:  multV
//			multiply and change a 3 vec:  multSetV
//			set to unit:  unit()
//			transform:  ???
//			toString()
//

/** A fairly conventional 3D matrix object that can transform sets of
    3D points and perform a variety of manipulations on the transform */
class D3Matrix {
	protected float c[][] = new float [3][3]; // Row, column
    static final double pi = 3.14159265;
    
    /** Create a new unit matrix */
    D3Matrix () {
    	for (int i=0; i<3; i++) 
    		c[i][i] = 1;
	    }
	
	// set matrix with three vectors
	// if rows is true, they are taken as the rows.
	// otherwise, taken as the columns
	D3Matrix( D3Vector xv, D3Vector yv, D3Vector zv, boolean rows)  {
		int i,j;
		if (rows) {
			for (i = 0; i<3; i++) {
				c[0][i] = xv.x[i];
				c[1][i] = yv.x[i];
				c[2][i] = zv.x[i];
				}
			}
		else {
			for (i = 0; i<3; i++) {
				c[i][0] = xv.x[i];
				c[i][1] = yv.x[i];
				c[i][2] = zv.x[i];
				}
			}
		} // end of constructor with vectors

	public void setValue(int i, int j, float value) {
		c[i][j] = value;
		}

	void setValue( D3Vector xv, D3Vector yv, D3Vector zv, boolean rows)  {
		int i,j;
		if (rows) {
			for (i = 0; i<3; i++) {
				c[0][i] = xv.x[i];
				c[1][i] = yv.x[i];
				c[2][i] = zv.x[i];
				}
			}
		else {
			for (i = 0; i<3; i++) {
				c[i][0] = xv.x[i];
				c[i][1] = yv.x[i];
				c[i][2] = zv.x[i];
				}
			}
		} // end of set value with vectors

	// set a row or column of the matrix
	void setValue (int index, float xv, float yv, float zv, boolean arow) {
		if (arow) {
			c[index][0] = xv;
			c[index][1] = yv;
			c[index][2] = zv;
			}
		else {
			c[0][index] = xv;
			c[1][index] = yv;
			c[2][index] = zv;
			}
		}
		
	// copy function for matrices
	void setValue(D3Matrix M) {
		int i,j;
		for (i=0; i<3; i++)
			for (j=0; j<3; j++)
				c[i][j] = M.c[i][j];
		}
	void zero() {  // set mat to 0 all entries
    	int i,j;
    	for (i=0; i<3; i++)
    		for (j=0; j<3; j++)
    			c[i][j] = 0;
	
	}
	float getValue(int i, int j) {
		return c[i][j];
		}
	
	D3Vector Row(int i) {
		D3Vector temp = new D3Vector(c[i][0], c[i][1], c[i][2]);
		return temp;
		}
		
	// Put row i into target
	void getRow(int i, D3Vector Target)  {
		Target.setValue(c[i][0], c[i][1], c[i][2]);
		}

	// Put col i into target
	void getColumn(int i, D3Vector Target)  {
		Target.setValue(c[0][i], c[1][i], c[2][i]);
		}

	// Put col i into target
	D3Vector Column(int i)  {
		D3Vector temp = new D3Vector(c[0][i], c[1][i], c[2][i]);
		return temp;
		}

    /** Scale by f in all dimensions */
    void scale(float fv) {
    	int i,j;
    	for (i=0; i<3; i++)
    		for (j=0; j<3; j++)
    			c[i][j] *= fv;
	    }
	    
    /** Multiply this matrix by a second: M = M*R */
    void Rmult(D3Matrix rhs) {
    
    	D3Matrix result = new D3Matrix();
    	D3Vector arow = new D3Vector();
    	int rowidx,column, i, j;
    	
    	result.zero();
    	for (rowidx=0; rowidx<3; rowidx++)  
    		for (column=0; column<3; column++) 
    			// Find the row, column entry
    			for (i=0; i<3; i++)
    				result.c[rowidx][column] += c[rowidx][i] * rhs.c[i][column];

		// Copy the result into this
    	for (i=0; i<3; i++)
    		for (j=0; j<3; j++)
				c[i][j] = result.c[i][j];
		

	    }  // end mult
    
    /** Multiply this matrix by a second: M = L*M */
    // Left sided multiplication.
    void Lmult(D3Matrix lhs) {
    
    	D3Matrix result = new D3Matrix();
    	int rowidx,column, i, j;
    	
    	
    	result.zero();
    	for (rowidx=0; rowidx<3; rowidx++)  
    		for (column=0; column<3; column++) 
    			// Find the row, column entry
    			for (i=0; i<3; i++)
    				result.c[rowidx][column] += lhs.c[rowidx][i] * c[i][column];

		// Copy the result into this
    	for (i=0; i<3; i++)
    		for (j=0; j<3; j++)
				c[i][j] = result.c[i][j];
		

	    }  // end Lmult
    

	
   /** rotate theta degrees about the y axis */
    void yrot(double theta) {
		theta *= (pi / 180);
		float ct = (float) Math.cos(theta);
		float st = (float) Math.sin(theta);

		// Set up the matrix and multiply on the left.
		D3Matrix rot = new D3Matrix();
		rot.setValue(0, ct, 0, st, false);
		rot.setValue(1, 0, 1, 0, false);
		rot.setValue(2, -st, 0, ct, false);
		
		Lmult(rot);
	    }

    /** rotate theta degrees about the x axis */
    void xrot(double theta) {
		theta *= (pi / 180);
		float ct = (float) Math.cos(theta);
		float st = (float) Math.sin(theta);

		// Set up the matrix and multiply on the left.
		D3Matrix rot = new D3Matrix();
		rot.setValue(0, 1,0,0, false);
		rot.setValue(1, 0, ct, st, false);
		rot.setValue(2, 0, -st, ct, false);
		
		Lmult(rot);
	    }
    /** rotate theta degrees about the z axis */
    void zrot(double theta) {
		theta *= (pi / 180);
		float ct = (float) Math.cos(theta);
		float st = (float) Math.sin(theta);

		// Set up the matrix and multiply on the left.
		D3Matrix rot = new D3Matrix();
		rot.setValue(0, ct, st, 0, false);
		rot.setValue(1, -st, ct, 0, false);
		rot.setValue(2, 0,0,1, false);
		
		Lmult(rot);
	    }
	    
	D3Vector mult(D3Vector v)  {
    	D3Vector temp = new D3Vector();
    	float sum = 0;
    	int i,j;
    	for (i = 0; i<3; i++) {
    		sum = 0;
       		for (j=0; j< 3; j++)
       			sum += c[i][j] * v.x[j];
       		temp.setValue(i,sum);
       		}
       	return temp;
       	}

     // Multiply a this matrix times a 3 vector and alter the 3 vector
    void multSetV(D3Vector v)  {
    	
    	D3Vector temp = new D3Vector();
    	float sum = 0;
    	int i,j;
    	for (i = 0; i<3; i++) {
    		sum = 0;
       		for (j=0; j< 3; j++)
       			sum += c[i][j] * v.x[j];
       		temp.setValue(i,sum);
       		}
       	v.setValue(temp);
		}  // end multSetV
	
		
   /** Reinitialize to the unit matrix */
    void unit() {
    	
    	int i,j; 
    	
		zero();
		for (i=0; i<3; i++)
			c[i][i] = 1;
	    }  // end unit

	// return true if matrix is identity.
	boolean isunit() {
		int i,j;
		for (i=0; i<3; i++) {
			if (c[i][i] != 1) return false;
			for (j = 0; j<3; j++) 
				if (i != j && c[i][j] != 0) return false;
			}
		return true;
		}
			

    public String toString() {
	return ("[" + c[0][0] + "," + c[0][1] + "," + c[0][2] + ";\n"
		 + c[1][0] + "," + c[1][1] + "," + c[1][2] + ";\n"
		 + c[2][0] + "," + c[2][1] + "," + c[2][2] + "]");
    }
        /** Scale along each row independently */
/*    void scale(float xf, float yf, float zf) {
    	int j;
		for (j=0; j<3; j++)
			c[0][j] *= fv;
		for (j=0; j<3; j++)
			c[1][j] *= fv;
		for (j=0; j<3; j++)
			c[2][j] *= fv;
    	c[0].scale(xf);
    	c[1].scale(yf);
    	c[2].scale(zf);
    	}
*/

/*    // Multiply a this matrix times a 3 vector
    D3Vector multV(D3Vector v)  {
    	
    	float vx = v.x, vy = v.y, vz = v.z;
    	float xt = x[0][0] * vx + xy * vy + xz * vz;
    	float yt = yx * vx + yy * vy + yz * vz;
    	float zt = zx * vx + zy * vy + zz * vz;

    	D3Vector temp = new D3Vector(xt, yt, zt);
    	
		return temp;
		}// end multV
*/		

    /** Transform nvert points from v into tv.  v contains the input
        coordinates in floating point.  Three successive entries in
	the array constitute a point.  tv ends up holding the transformed
	points as integers; three successive entries per point 
    void transform(float v[], int tv[], int nvert) {
		float lxx = xx, lxy = xy, lxz = xz;
		float lyx = yx, lyy = yy, lyz = yz;
		float lzx = zx, lzy = zy, lzz = zz;
		for (int i = nvert * 3; (i -= 3) >= 0;) {
		    float x = v[i];
		    float y = v[i + 1];
		    float z = v[i + 2];
		    tv[i    ] = (int) (x * lxx + y * lxy + z * lxz );
		    tv[i + 1] = (int) (x * lyx + y * lyy + z * lyz);
		    tv[i + 2] = (int) (x * lzx + y * lzy + z * lzz );
			}
	    } // end transform
*/



}  // end of class D3Matrix


