/* THIS FILE MODIFIED BY DANIEL TALAYCO FROM:
 * Sun's Matrix3D.java	1.2 96/12/06
 * It no longer bears any resemblance to the original.
 */

// Modified by Dan Talayco, 2/98
// For vector and matrix 4 D classes.
//
//
// Class D4Vector:  A 4 dimensional vector class.
// Interface:
//	Constructor:  Default as 0 vector
//		With three float values for components
//
// 	Operations:
//      def constructor as 0 vector
//	    constructor with 4 floats given
//		dot product:  dot
//		add another vector to this one:  addto  
//		cross product:  cross
//		norm
//		scale by factor:  scale
//		Convert to array:   toArray
//	

class D4Vector {
	protected float x[] = {0,0,0,0};   // components
	
	// Constructors
	D4Vector () {
		}
	D4Vector (float x0, float x1, float x2, float x3) {
		x[0] = x0; x[1] = x1; x[2] = x2; x[3] = x3;
		}
	
	D4Vector (D4Vector v) {  // copy constructor
		setValue(v);
		}

	boolean equals (D4Vector rhs) {
		for (int i=0; i<4; i++)
			if (x[i] != rhs.x[i])
				return false;
		return true;
		}

	void setValue (float x0, float x1, float x2, float x3) {
		x[0] = x0; x[1] = x1; x[2] = x2; x[3] = x3;
		}
		
	void setValue (int index, float val) {
		x[index] = val;
		}

	void setValue (D4Vector xv) {
		for (int i=0; i<4; i++)
			x[i] = xv.x[i];
		}

	void setValue (D4iVector xv) {
		for (int i=0; i<4; i++)
			x[i] = xv.getValue(i);
		}
		
	void setValue (float [] v) {
		for (int i=0; i<4; i++)
			x[i] = v[i];
		}

	float getValue(int index) {
		return x[index];
		}

	synchronized double norm() {
		double sum = 0;
		for (int i=0; i<4; i++) 
			sum += x[i]*x[i];
		return Math.sqrt(sum);
		}
	
	// Return sum
	D4Vector add(D4Vector Rhs) {
		D4Vector temp = new D4Vector(Rhs);
		addto(temp);
		return temp;
	
	}


	void addto (D4Vector Rhs) {
		for (int i=0; i<4; i++)
			x[i] += Rhs.x[i];
		}
		
	float dot(D4Vector Rhs) {
		float sum=0;
		for (int i=0; i<4; i++)
			sum += x[i] * Rhs.x[i];
		return sum;
		}
	
		
    void scale(float fv) {
    	for(int i=0; i<4; i++)
    		x[i] *= fv;
		}

	// Project from a 4 vector to a 3 vector.  
	// Remove the index passed as a parameter.
	D3Vector projection(int index)  {
		D3Vector temp = new D3Vector ();
		int i,j=0;
		for ( i=0; i<4; i++)  
			if (i != index) temp.setValue(j++,x[i]);
		return temp;
		} // End of projection function.
		
	// Project from a 4 vector to a 3 vector.  
	// Project the three indices passed in the order passed.
	D3Vector projection(int i0, int i1, int i2)  {
		D3Vector temp = new D3Vector (x[i0],x[i1],x[i2]);
		return temp;
		} // End of projection function.
		
	// Project from a 4 vector to a 3 vector.  
	// Remove the index passed as a parameter.
	void projection(int index, D3Vector target)  {
		int i,j=0;
		for ( i=0; i<4; i++)  
			if (i != index) target.setValue(j++,x[i]);
		} // End of projection function.
		
	// Project from a 4 vector to a 3 vector.  
	// Project the three indices passed in the order passed.
	void projection(int i0, int i1, int i2, D3Vector target)  {
		target.setValue(x[i0],x[i1],x[i2]);
		} // End of projection function.
		
 	
    public String toString() {
		return ("["  + x[0] + "," + x[1] + "," + x[2] + "," + x[3] + "]");
	    }

}  // end of D4Vector




// Class D4Matrix:  A 4 dimensional matrix class.
// Interface:
// 		Constructor:  Default as identity
// 			With three vectors as columns or rows (determined by boolean param).
//			scale by one value or scale x, y, z columns independently
//			rotate about x, y, z:  xrot, yrot, zrot
//			multiply a 4 vec and return 4 vec:  multV
//			multiply and change a 4 vec:  multSetV
//			set to unit:  unit()
//			transform:  ???
//			toString()
//
class D4Matrix {
	protected float c[][] = new float [4][4]; // Row, column
    static final double pi = 3.14159265;
    
    // Create a new unit matrix 
    D4Matrix () {
    	for (int i=0; i<4; i++) 
    		c[i][i] = 1;
	    }
	
	// set matrix with three vectors
	// if rows is true, they are taken as the rows.
	// otherwise, taken as the columns
	D4Matrix( D4Vector x0, D4Vector x1, D4Vector x2,  D4Vector x3, boolean rows)  {
		int i,j;
		if (rows) {
			for (i = 0; i<4; i++) {
				c[0][i] = x0.x[i];
				c[1][i] = x1.x[i];
				c[2][i] = x2.x[i];
				c[3][i] = x3.x[i];
				}
			}
		else {
			for (i = 0; i<4; i++) {
				c[i][0] = x0.x[i];
				c[i][1] = x1.x[i];
				c[i][2] = x2.x[i];
				c[i][3] = x3.x[i];
				}
			}
		} // end of constructor with vectors

	void setValue(int i, int j, float value) {
		c[i][j] = value;
		}

	void setValue(D4Vector x0, D4Vector x1, D4Vector x2,  D4Vector x3, boolean rows)  {
		int i,j;
		if (rows) {
			for (i = 0; i<4; i++) {
				c[0][i] = x0.x[i];
				c[1][i] = x1.x[i];
				c[2][i] = x2.x[i];
				c[3][i] = x3.x[i];
				}
			}
		else {
			for (i = 0; i<4; i++) {
				c[i][0] = x0.x[i];
				c[i][1] = x1.x[i];
				c[i][2] = x2.x[i];
				c[i][3] = x3.x[i];
				}
			}
		} // end of set value with vectors

	// set a row or column of the matrix
	void setValue (int index, float x0, float x1, float x2, float x3, boolean arow) {
		if (arow) {
			c[index][0] = x0;
			c[index][1] = x1;
			c[index][2] = x2;
			c[index][3] = x3;
			}
		else {
			c[0][index] = x0;
			c[1][index] = x1;
			c[2][index] = x2;
			c[3][index] = x3;
			}
		}
	
	void zero() {  // set mat to 0 all entries
    	int i,j;
    	for (i=0; i<4; i++)
    		for (j=0; j<4; j++)
    			c[i][j] = 0;
	
	}
	float getValue(int i, int j) {
		return c[i][j];
		}
	
	D4Vector Row(int i) {
		D4Vector temp = new D4Vector(c[i][0], c[i][1], c[i][2], c[i][3]);
		return temp;
		}
		
	// Put row i into target
	void getRow(int i, D4Vector Target)  {
		Target.setValue(c[i][0], c[i][1], c[i][2], c[i][3]);
		}

	// Put col i into target
	void getColumn(int i, D4Vector Target)  {
		Target.setValue(c[0][i], c[1][i], c[2][i], c[3][i]);
		}

	// Put col i into target
	D4Vector Column(int i)  {
		D4Vector temp = new D4Vector(c[0][i], c[1][i], c[2][i], c[3][i]);
		return temp;
		}

    // Scale by f in all dimensions 
    void scale(float fv) {
    	int i,j;
    	for (i=0; i<4; i++)
    		for (j=0; j<4; j++)
    			c[i][j] *= fv;
	    }
	    
    // Multiply this matrix by a second: M = M*R 
    void Rmult(D4Matrix rhs) {
    
    	D4Matrix result = new D4Matrix();
    	D4Vector arow = new D4Vector();
    	int rowidx,column, i, j;
    	
    	result.zero();
    	for (rowidx=0; rowidx<4; rowidx++)  
    		for (column=0; column<4; column++) 
    			// Find the row, column entry
    			for (i=0; i<4; i++)
    				result.c[rowidx][column] += c[rowidx][i] * rhs.c[i][column];

		// Copy the result into this
    	for (i=0; i<4; i++)
    		for (j=0; j<4; j++)
				c[i][j] = result.c[i][j];
		

	    }  // end mult
    
    // Multiply this matrix by a second: M = L*M 
    // Left sided multiplication.
    void Lmult(D4Matrix lhs) {
    
    	D4Matrix result = new D4Matrix();
    	int rowidx,column, i, j;
    	
    	
    	result.zero();
    	for (rowidx=0; rowidx<4; rowidx++)  
    		for (column=0; column<4; column++) 
    			// Find the row, column entry
    			for (i=0; i<4; i++)
    				result.c[rowidx][column] += lhs.c[rowidx][i] * c[i][column];

		// Copy the result into this
    	for (i=0; i<4; i++)
    		for (j=0; j<4; j++)
				c[i][j] = result.c[i][j];
		

	    }  // end Lmult
    
	// Set the rotation values for axis1 and axis2.  ct is cos t,
	// st is sin t.  General format is
	// ct -st
	// st  ct
	//
	// PRECONDITION:  this is unit matrix in other coordinates
	void SetRotAxes(int axis1, int axis2, float ct, float st) {
		c[axis1][axis1] = ct;
		c[axis2][axis2] = ct;
		c[axis1][axis2] = -st;
		c[axis2][axis1] = st;
		}  

    // rotate theta degrees about the x axis 
    void x0rot(double theta) {
		theta *= (pi / 180);
		float ct = (float) Math.cos(theta);
		float st = (float) Math.sin(theta);

		// Set up the matrix and multiply on the left.
		D4Matrix rot = new D4Matrix();
		rot.SetRotAxes(1,2,ct,st);
		
		Lmult(rot);
	    }
	    
	
	// The next three routines are rotations with the x3 axis fixed
	// and then rotating about another axis.  
	// See below for a more general rotation routine.
	//
    // rotate theta degrees about the y axis 
    void x1rot(double theta) {
		theta *= (pi / 180);
		float ct = (float) Math.cos(theta);
		float st = (float) Math.sin(theta);

		// Set up the matrix and multiply on the left.
		D4Matrix rot = new D4Matrix();
		rot.SetRotAxes(2,0,ct,st);
		
		Lmult(rot);
	    }

   // rotate theta degrees about the z axis 
    void x2rot(double theta) {
		theta *= (pi / 180);
		float ct = (float) Math.cos(theta);
		float st = (float) Math.sin(theta);

		// Set up the matrix and multiply on the left.
		D4Matrix rot = new D4Matrix();
		rot.SetRotAxes(0,1,ct,st);
		
		Lmult(rot);
	    }
	    
	// rotate theta degrees with the given axes moving.
	// Normal rotations are xy, zx, yz (for order, for ccw.)   
	// Still unclear for fourth axis (3/7)
    void Rotate(double theta, int axis1, int axis2) {
		theta *= (pi / 180);
		float ct = (float) Math.cos(theta);
		float st = (float) Math.sin(theta);

		// Set up the matrix and multiply on the left.
		D4Matrix rot = new D4Matrix();
		rot.SetRotAxes(axis1, axis2,ct,st);
		
		Lmult(rot);
	    }
	    
	D4Vector mult(D4Vector v)  {
    	D4Vector temp = new D4Vector();
    	float sum = 0;
    	int i,j;
    	for (i = 0; i<4; i++) {
    		sum = 0;
       		for (j=0; j< 4; j++)
       			sum += c[i][j] * v.x[j];
       		temp.setValue(i,sum);
       		}
       	return temp;
       	}

     // Multiply a this matrix times a 4 vector and alter the 4 vector
    void multSetV(D4Vector v)  {
    	
    	D4Vector temp = new D4Vector();
    	float sum = 0;
    	int i,j;
    	for (i = 0; i<4; i++) {
    		sum = 0;
       		for (j=0; j< 4; j++)
       			sum += c[i][j] * v.x[j];
       		temp.setValue(i,sum);
       		}
       	v.setValue(temp);
		}  // end multSetV
	
		
   // Reinitialize to the unit matrix 
    void unit() {
    	
    	int i,j; 
    	
		zero();
		for (i=0; i<4; i++)
			c[i][i] = 1;
	    }  // end unit

	// return true if matrix is identity.
	boolean isunit() {
		int i,j;
		for (i=0; i<4; i++) {
			if (c[i][i] != 1) return false;
			for (j = 0; j<4; j++) 
				if (i != j && c[i][j] != 0) return false;
			}
		return true;
		}
	// Project this matrix to a 3x3 matrix removing teh
	// indicated axis.  There are clever ways to do this,
	// but it I'm doing it pretty directly.
	// PRECONDITION:  axis is between 0 and 3.
	void projection(int axis, D3Matrix target) {
	
		int i,j;  // row and column for this
		int row=0, column=0; // for target
		for (i=0; i<4; i++)
			if (i != axis) {
				for (j=0; j<4; j++)
					if (j != axis) 
						target.setValue(row, column++, c[i][j]);
				column = 0;
				row += 1;
				}
		}
			
    public String toString() {
	return ("[" + c[0][0] + "," + c[0][1] + "," + c[0][2] + "," + c[0][3] + ";\n"
		 + c[1][0] + "," + c[1][1] + "," + c[1][2] + "," + c[1][3] + ";\n"
		 + c[2][0] + "," + c[2][1] + "," + c[2][2] + "," + c[2][3] + "\n"
		 + c[3][0] + "," + c[3][1] + "," + c[3][2] + "," + c[3][3] + "]");
    }

}  // end of class D4Matrix


